// RW Bumpmaster
// ==============
// Imageboard Retrieval & Archiving Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/curator/bumpmaster/bumpmaster.hpp
// -Declarations of the baseline RW Bumpmaster sub-library

#pragma once

#include "board.hpp"
#include "catalog.hpp"
#include "page.hpp"
#include "post.hpp"
#include "site.hpp"
#include "thread.hpp"

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
class Bumpmaster_b : public RW {
 public:
  Bumpmaster_b();                                   // default ctor
  explicit Bumpmaster_b(std::string const& handle,  // tag-parm'd ctor
                        Type const         type);
};

//------------------------------------------------------------------------------
class Bumpmaster : public Bumpmaster_b {
 public:
  Bumpmaster();                                    // default ctor
  explicit Bumpmaster(std::string const& handle);  // tag-parm'd ctor

  // void do_preflight(); wrapper for public, controlled access to priv func

  inline auto board() const -> Board& { return board_; }
  inline auto catalog() const -> Catalog& { return catalog_; }
  inline auto dollhouse() const -> Dollhouse& { return dollhouse_; }
  inline auto page() const -> Page& { return page_; }
  inline auto post() const -> Post& { return post_; }
  inline auto site() const -> Site& { return site_; }
  inline auto thread() const -> Thread& { return thread_; }

 private:
  /**
   * @note also binds all member slave objs to this master
   */
  void preflight();

  unique_ptr<Board_b>   _board_{make_unique<Board_b>("bumpmaster_board")};
  unique_ptr<Catalog_b> _catalog_{make_unique<Catalog_b>("bumpmaster_catalog")};
  unique_ptr<Dollhouse_b> _dollhouse_{
      make_unique<Dollhouse_b>("bumpmaster_dollhouse")};
  unique_ptr<Page_b>   _page_{make_unique<Page_b>("bumpmaster_page")};
  unique_ptr<Post_b>   _post_{make_unique<Post_b>("bumpmaster_post")};
  unique_ptr<Site_b>   _site_{make_unique<Site_b>("bumpmaster_site")};
  unique_ptr<Thread_b> _thread_{make_unique<Thread_b>("bumpmaster_thread")};

  Board&     board_{reference_to<Board>(_board_.get())};
  Catalog&   catalog_{reference_to<Catalog>(_catalog_.get())};
  Dollhouse& dollhouse_{reference_to<Dollhouse>(_dollhouse_.get())};
  Page&      page_{reference_to<Page>(_page_.get())};
  Post&      post_{reference_to<Post>(_post_.get())};
  Site&      site_{reference_to<Site>(_site_.get())};
  Thread&    thread_{reference_to<Thread>(_thread_.get())};
};

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v211204
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
