//  RW Dollhouse
// ==============
// Docking, Charging-System, Storage, and General Outboard Systems Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/dollhouse/dollhouse.hpp
// -Declarations of the baseline RW Dollhouse library

#pragma once

#include "incls_all.hpp"

namespace rw::dollhouse {

//------------------------------------------------------------------------------
class Dollhouse_b : public RW {
 public:
  Dollhouse_b();                                    // default ctor
  explicit Dollhouse_b(std::string const& handle);  // tag-parm'd ctor

 protected:
  ///////////////
  // New 'RW Wired' reworks...
  //
  // DESIGN:
  // -As a simple, basic control mechanism for starters we'll use some
  // protected/private classes and a few functions from here within Dollhouse.
  // Later, we can add more access restrictions and some further data
  // validations; improving things bit-by-bit as we go along, using just a few
  // of the straightforward language features readily available to us.
  ///////////////

  // DESIGN:
  // - add work-stealing queues of Dolldrop 's
  // - use S.Parent's libdispatch-based async
  // - this is actually a general need, so add it to the RW Foundations instead

  class Dolldrop {
   public:
    //

    // DESIGN:
    // void push_drop() {}
    // void yank_drop() {}

    // TEMP:
    // for now, we'll just use basic accessors w/ no validations or gatekeeping
    auto ask_drop(Hip const& tag, std::string_view const uri) -> bool {
      // TEST:
      ostringstream ss{};
      ss << "(requested pull drop: '" << uri << "')\n"
         << "asked to pull a drop:  \t\t";
      prt_tag(tag, ss.str());
      //

      // DESIGN: TODO:
      // return do_atomiconn(
      //   lockout();
      //     connect();
      //       do_req(uri);
      //     disconnect();
      //   unlockout();
      // );

      // return false;  // TEMP:
      return true;  // TEMP:
    }

    auto get_drop(Hip const& tag) const -> std::string_view const {
      // TEST:
      prt_tag(tag, "asked for drop payload: \t");

      return "foo_9001";  // TEMP:
    }

   private:
    class Inbox {
      //
     private:
      string data_;
    } inbox;

    class Outbox {
      //
     private:
      string data_;
    } outbox;

  } dolldrop;

 private:
  ///

  // TEMP:
  // for now, we'll just use basic accessors w/ no validations or gatekeeping
  //

  int const cnt{4};
  //
  // DESIGN: rm code-envy
  //
  auto connect() -> bool {
    int conn_atts{cnt};

    while (true) {
      try {
        //
        if (conn_atts)
          return wired.connect();
        else
          error("connection attempts exceeded");

      } catch (...) {
        conn_atts--;
        puts("retrying connection...");
      }
    }
    return false;  // shouldn't reach
  }
  //
  auto disconnect() -> bool {
    int dconn_atts{cnt};

    while (true) {
      try {
        //
        if (dconn_atts)
          return wired.disconnect();
        else
          error("disconnection attempts exceeded");

      } catch (...) {
        dconn_atts--;
        puts("retrying disconnection...");
      }
    }
    return false;  // shouldn't reach
  }

  class Wired {
   public:
    //

    bool connect() {
      return true;  // TEMP:
    }

    bool disconnect() {
      return true;  // TEMP:
    }

    // DESIGN:
    // Bumpmaster specific:
    //
    // void supply_text() {}
    // void supply_images() {}
    // void supply_captcha() {}
    // void signal_captcha() {}  // use callback into a graphicom obj
    // void captcha_image() {}   //  "

   private:
    class Curly {
      // DESIGN: just wrap cURL calls with curlcpp functions from here. We'll
      // deal primarily with variations of std::string 's for Dolldrop payloads,
      // which curlcpp supports quite well already.
      //

     private:
      string data_;
    } curly;

  } wired;
};

//------------------------------------------------------------------------------
class Dollhouse : public Dollhouse_b {
 public:
  Dollhouse();                                    // default ctor
  explicit Dollhouse(std::string const& handle);  // tag-parm'd ctor

  ///

  // shell & power care
  void dock_me() {}
  void charge_me() {}
  void sleep_me() {}
  void wake_me() {}
  void store_me() {}

  ///

  // New 'RW Wired' reworks...
  //
  // TEMP:
  // for now, we'll just use basic accessors w/ no validations or gatekeeping
  //
  auto ask_drop(Hip const& tag, std::string_view const uri) -> bool {
    return dolldrop.ask_drop(tag, uri);
  }
  //
  auto get_drop(Hip const& tag) const -> std::string_view const {
    return dolldrop.get_drop(tag);
  }

  ///

  inline auto electro() const -> Electro& { return electro_; }
  inline auto shell() const -> Shell& { return shell_; }

 private:
  unique_ptr<Electro_b> _electro_{make_unique<Electro_b>("dollhouse_electro")};
  unique_ptr<Shell_b>   _shell_{make_unique<Shell_b>("dollhouse_shell")};

  Electro& electro_{reference_to<Electro>(_electro_.get())};
  Shell&   shell_{reference_to<Shell>(_shell_.get())};
};

}  // namespace rw::dollhouse

//    --=^=--

// "Now be careful. If you go for a full charge, it can get up there."

// rw_dollhouse_lib.v211204
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
