//  RW Foundations
// ================
// The RW Foundational Libraries

// Filename: ./include/rw/rw.hpp
// -Declarations of the baseline RW library

#pragma once

#include "incls_all.hpp"

namespace rw {

//------------------------------------------------------------------------------
/** The master base class from which all other tagged rw:: classes should derive
 * @note:
 */
class RW {
 public:
  /** Construct a @c RW object
   * @note default ctor
   */
  RW();

  /**
   * @param tag
   */
  RW(std::string const& handle_tag);
  //
  RW(std::string const& handle_tag, Type const type);

  ~RW();

  /** RW object's Hip @a handle (it's 'name tag')
   * @return the object's handle
   */
  auto hdl() const -> std::string;

  /** RW object's Hip @a ID
   * @return the object's id
   */
  auto id() const -> unsigned int;

  /** RW object's Hip @a address
   * @return the object's address (it's pointer)
   */
  auto addr() const -> RW*;

  /** RW object's Hip @a type
   * @return
   */
  auto typ() const -> Type;

  /** RW object's Hip @a 'tag' (the full data structure)
   * @note: sending/receiving these is our primary means for tracing callgraphs
   * @return the object's Hip
   */
  auto t() const -> Hip const&;

 private:
  /** Incremented by one for each new instance of this base class (when created)
   */
  static unsigned long rw_id;

  /** ID's & ptrs are unique, but handles may be identical among the same types.
   * @note: eg; {sumomo.0}, {sumomo.1}, {arigato.2}, {dollhouse.3}, {arigato.4}
   */
  Hip const hip_;
};

}  // namespace rw

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v211204
// Copyright (2021)
// License MIT(Expat) https://opensource.org/licenses/MIT
