//   RW Board
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./src/curator/bumpmaster/board.cpp
// -Definitions of the baseline RW Board sub-library

#pragma once

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
Board_b::Board_b() : Board_b{"board_b_obj"} {}

//------------------------------------------------------------------------------
Board_b::Board_b(std::string const& handle)
    : RW{handle},            //
      do_stamps_{do_stamps}  // member field allows individual class' setting
{}

//------------------------------------------------------------------------------
void Board_b::bind_master(Hip const& tag) {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"bind_master()", tag, Call_clk::now()});

  if (tag.type_of(Type::Bumpmaster)) {
    if (master_.ptr == nullptr)  // this slave object not yet bound to a master,
      master_ = tag;             // accept the caller as Master
    else
      error("I already belong to Master", t(), "Board_b::bind_master");

  } else
    error("I can only accept a 'Bumpmaster' as Master", t(),
          "Board_b::bind_master()");
}

//------------------------------------------------------------------------------
Board::Board() : Board{"board_obj"} {}

//------------------------------------------------------------------------------
Board::Board(std::string const& handle) : Board_b{handle} {}

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v211204
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
