//   RW Catalog
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./src/curator/bumpmaster/catalog.cpp
// -Definitions of the baseline RW Catalog sub-library

#pragma once

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
Catalog_b::Catalog_b() : Catalog_b{"catalog_b_obj"} {}

//------------------------------------------------------------------------------
Catalog_b::Catalog_b(std::string const& handle)
    : RW{handle},            //
      do_stamps_{do_stamps}  // member field allows individual class' setting
{}

//------------------------------------------------------------------------------
auto Catalog_b::parse_threads(Hip const& tag /*, std::vector<Thread> threads*/)
    -> int {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"parse_threads()", tag, Call_clk::now()});

  if (tag == master_) {
    // TODO:
    // ...
    // threads.emplace_back(thrd);
    // ...
  } else
    only_master_can(master_, tag, t(), "Catalog_b::parse_threads()");

  return 1;  // TEMP:
  // return threads.size();
}

//------------------------------------------------------------------------------
auto Catalog_b::have_new(Hip const& tag /*, std::vector<Thread> threads*/)
    -> bool {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"have_new()", tag, Call_clk::now()});

  if (tag == master_) {
    // TODO:
    // ...
  } else
    only_master_can(master_, tag, t(), "Catalog_b::have_new()");

  return true;  // TEMP:
}

//------------------------------------------------------------------------------
void Catalog_b::bind_master(Hip const& tag) {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"bind_master()", tag, Call_clk::now()});

  if (tag.type_of(Type::Bumpmaster)) {
    if (master_.ptr == nullptr)  // this slave object not yet bound to a master,
      master_ = tag;             // accept the caller as Master
    else
      error("I already belong to Master", t(), "Catalog_b::bind_master");

  } else
    error("I can only accept a 'Bumpmaster' as Master", t(),
          "Catalog_b::bind_master()");
}

//------------------------------------------------------------------------------
Catalog::Catalog() : Catalog{"catalog_obj"} {}

//------------------------------------------------------------------------------
Catalog::Catalog(std::string const& handle) : Catalog_b{handle} {}

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v211204
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
