//   RW Thread
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./src/curator/bumpmaster/thread.cpp
// -Definitions of the baseline RW Thread sub-library

#pragma once

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
Thread_b::Thread_b() : Thread_b{"thread_b_obj"} {}

//------------------------------------------------------------------------------
Thread_b::Thread_b(std::string const& handle)
    : RW{handle},            //
      do_stamps_{do_stamps}  // member field allows individual class' setting
{}

//------------------------------------------------------------------------------
auto Thread_b::parse_posts(const Hip& tag /*, std::vector<Thread> posts*/)
    -> int {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"parse_posts()", tag, Call_clk::now()});

  if (tag == master_) {
    // TODO:
    // ...
    // posts.emplace_back(thrd);
    // ...
  } else
    only_master_can(master_, tag, t(), "Thread_b::parse_posts()");

  return 1;  // TEMP:
  // return posts.size();
}

//------------------------------------------------------------------------------
auto Thread_b::crosslink_posts(Hip const& tag) -> bool {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"crosslink_posts()", tag, Call_clk::now()});

  if (tag == master_) {
    // TODO:
    // ...
  } else
    only_master_can(master_, tag, t(), "Thread_b::crosslink_posts()");

  return true;  // TEMP:
}

//------------------------------------------------------------------------------
void Thread_b::bind_master(Hip const& tag) {
  if (do_stamps_)
    stamps_.emplace(Callstamp{"bind_master()", tag, Call_clk::now()});

  if (tag.type_of(Type::Bumpmaster)) {
    if (master_.ptr == nullptr)  // this slave object not yet bound to a master,
      master_ = tag;             // accept the caller as Master
    else
      error("I already belong to Master", t(), "Thread_b::bind_master");

  } else
    error("I can only accept a 'Bumpmaster' as Master", t(),
          "Thread_b::bind_master");
}

//------------------------------------------------------------------------------
Thread::Thread() : Thread{"thread_obj"} {}

//------------------------------------------------------------------------------
Thread::Thread(std::string const& handle) : Thread_b{handle} {}

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v211204
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
